import sys
import platform
import time
import socket
import re
import urllib2
import base64

import usb.core
import usb.util
import struct
import Operations as op

import thread
import threading
import time
import numpy as np

DEVICE = None
DEVICE_TYPE = None
epBulkWriter = None
epBulkReader = None

class Monsoon(object):
    def __init__(self, *args, **kwargs):
        self._statusPacket = op.statusPacket
        self.fineThreshold = 64000
        self.auxFineThreshold = 30000
        self.mainvoltageScale = 4
        self.usbVoltageScale = 2
        self.ADCRatio = (float)(62.5 / 1e6); #Each tick of the ADC represents this much voltage

        self.FactoryRes = 0.050
        self.auxFactoryRes = 0.1
        self.padding = np.zeros(64)
        pass

    def setup_usb(self):
        global DEVICE 
        global DEVICE_TYPE
        global epBulkWriter
        global epBulkReader

        DEVICE = usb.core.find(idVendor=0x2AB9, idProduct=0x0001)
        # On Linux we need to detach usb HID first
        if "Linux" == platform.system():
            try:
                DEVICE.detach_kernel_driver(0)
            except:
                pass # already unregistered    

        DEVICE.set_configuration()
        cfg = DEVICE.get_active_configuration()
        intf = cfg[(0,0)]

        epBulkWriter = usb.util.find_descriptor(
            intf,
            custom_match = \
                lambda e: \
                usb.util.endpoint_direction(e.bEndpointAddress) == \
                usb.util.ENDPOINT_OUT)
        epBulkReader = usb.util.find_descriptor(
            intf,
            custom_match = \
                lambda e: \
                usb.util.endpoint_direction(e.bEndpointAddress) == \
                usb.util.ENDPOINT_IN)



        #notes are poorly documented, values are:
        #ctrl_transfer(bmRequestType, bmRequest, wValue, wIndex)
    def sendCommand(self,operation, value):
        try:
            value = int(value)
            value_array = struct.unpack("4b",struct.pack("I",value))
            operation_array = struct.unpack("4b",struct.pack("I",operation))
            wValue = struct.unpack("H",struct.pack("bb",value_array[0],value_array[1]))[0]
            wIndex = struct.unpack("H",struct.pack("bb",operation_array[0],value_array[2]))[0]
            DEVICE.ctrl_transfer(op.Control_Codes.USB_OUT_PACKET,op.Control_Codes.USB_SET_VALUE,wValue,wIndex,"0",5000)
        except:
            print("Control Transfer Error")
    
    def sendStopCommand(self):
        DEVICE.ctrl_transfer(op.Control_Codes.USB_IN_PACKET,op.Control_Codes.USB_REQUEST_STOP,0,0,0,5000)

    def getValue(self,operation,valueLength):
        operation_array = struct.unpack("4b",struct.pack("I",operation))
        wIndex = struct.unpack("H",struct.pack("bb",operation_array[0],0))[0]
        result = DEVICE.ctrl_transfer(op.Control_Codes.USB_IN_PACKET,op.Control_Codes.USB_SET_VALUE,0,wIndex,4,5000)
        result = struct.unpack("I",result)[0]
        return result

    def amps_from_raw(self,raw):
        raw = min(raw,1023)
        result = 8.0*(1.0-(raw/1023.0))
        return result

    def raw_from_amps(self,value):
        result = 65535 - 0x0F00 * (value / 65535) + 0x0F00
        return result

    def setVout(self,value):
        vout = value * op.Conversion.FLOAT_TO_INT
        self.sendCommand(op.OpCodes.setHVMainVoltage,vout) 
    def setPowerupTime(self,value):
        self.sendCommand(op.OpCodes.setPowerupTime,value)
    def setPowerUpCurrentLimit(self, value):
        value = self.raw_from_amps(value)
        self.sendCommand(op.OpCodes.SetHVPowerUpCurrentLimit,value)
    def setRunTimeCurrentLimit(self, value):
        value = self.raw_from_amps(value)
        self.sendCommand(op.OpCodes.SetHVRunCurrentLimit,value)
    def setMainFineScale(self, value):
        self.sendCommand(op.OpCodes.setMainFineScale,value)
    def setMainCoarseScale(self,value):
        self.sendCommand(op.OpCodes.setMainCoarseScale,value)
    def setUSBFineScale(self, value):
        self.sendCommand(op.OpCodes.setUSBFineScale,value)
    def setUSBCoarseScale(self,value):
        self.sendCommand(op.OpCodes.setUSBCoarseScale,value)
    def setAuxFineScale(self,value):
        self.sendCommand(op.OpCodes.setAuxFineScale,value)
    def setAuxCoarseScale(self,value):
        self.sendCommand(op.OpCodes.setAuxCoarseScale,value)

    def setMainFineZeroOffset(self, value):
        self.sendCommand(op.OpCodes.SetMainFineZeroOffset,value)
    def setMainCoarseZeroOffset(self,value):
        self.sendCommand(op.OpCodes.SetMainCoarseZeroOffset,value)
    def setUSBFineZeroOffset(self, value):
        self.sendCommand(op.OpCodes.SetUSBFineZeroOffset,value)
    def setUSBCoarseZeroOffset(self,value):
        self.sendCommand(op.OpCodes.SetUSBCoarseZeroOffset,value)
    def setUSBPassthroughMode(self, USBPassthroughCode):
        self.sendCommand(op.OpCodes.setUsbPassthroughMode,USBPassthroughCode)
    def setVoltageChannel(self, VoltageChannelCode):
        self.sendCommand(op.OpCodes.setHVVoltageChannel,value)

    def getSerialNumber(self):
        serialNumber = self.getValue(op.OpCodes.SetSerialNumber,2)
        return serialNumber
    def getVoltageChannel(self):
        return(self.getValue(op.OpCodes.setHVVoltageChannel,1))

    def StartSampling(self):
        self.Start(1250,0xFFFF)

    def fillStatusPacket(self):
        self._statusPacket.mainFineScale = float(self.getValue(op.OpCodes.setMainFineScale,2))
        self._statusPacket.mainCoarseScale = float(self.getValue(op.OpCodes.setMainCoarseScale,2)) 
        self._statusPacket.usbFineScale = float(self.getValue(op.OpCodes.setUSBFineScale,2)) 
        self._statusPacket.usbCoarseScale = float(self.getValue(op.OpCodes.setUSBCoarseScale,2))
        self._statusPacket.auxFineScale = float(self.getValue(op.OpCodes.setAuxFineScale,2))
        self._statusPacket.auxCoarseScale = float(self.getValue(op.OpCodes.setAuxCoarseScale,2))

        self._statusPacket.mainFineZeroOffset = float(self.getValue(op.OpCodes.SetMainFineZeroOffset,2))
        self._statusPacket.mainCoarseZeroOffset = float(self.getValue(op.OpCodes.SetMainCoarseZeroOffset,2))
        self._statusPacket.usbFineZeroOffset = float(self.getValue(op.OpCodes.SetUSBFineZeroOffset,2))
        self._statusPacket.usbCoarseZeroOffset = float(self.getValue(op.OpCodes.SetUSBCoarseZeroOffset,2))

        #ctrl_transfer(bmRequestType, bmRequest, wValue, wIndex)
    def Start(self,calTime, maxTime):
        self.fillStatusPacket()
        value_array = struct.unpack("4B",struct.pack("I",calTime))
        maxtime_array = struct.unpack("4B",struct.pack("I",maxTime))
        wValue = struct.unpack("H",struct.pack("BB",value_array[0],value_array[1]))[0]
        wIndex = struct.unpack("H",struct.pack("BB",op.Control_Codes.USB_REQUEST_START,0))[0]

        DEVICE.ctrl_transfer(op.Control_Codes.USB_OUT_PACKET,op.Control_Codes.USB_REQUEST_START,wValue,wIndex,maxtime_array,1000)

    def BulkRead(self):
        return(DEVICE.read(0x81,64,timeout=0))

    def swizzlePacket(self, packet):
        length = len(packet)
        packet = np.array(packet)
        evenBytes = packet[4::2]
        oddBytes = packet[5::2]
        swizzledBytes = np.insert(evenBytes,np.arange(len(oddBytes)),oddBytes)
        swizzledPacket = np.hstack([packet[0:4],swizzledBytes[:],self.padding[0:(58-length)]])
        swizzledPacket = np.array(swizzledPacket).astype('B')
        packetLength = len(swizzledPacket)
        rawBytes = struct.pack('B'*packetLength,*swizzledPacket)
        measurements = struct.unpack("HBBHHHHHHHHBBHHHHHHHHBBHHHHHHHHBB",rawBytes)
        measurements = list(measurements)
        return measurements





